@@ -4,7 +4,7 @@ use 5.006001;
 use strict;
 use warnings;
 
-our $VERSION = '2.002001';
+our $VERSION = '2.005001';
 $VERSION =~ tr/_//d;
 
 use Carp;
@@ -415,6 +415,24 @@ sub _acmp {
     length($xstr) <=> length($ystr) || $xstr cmp $ystr;
 }
 
+sub _scmp {
+    # Compare two signed values. Return -1, 0, or 1.
+    my ($class, $xa, $xs, $ya, $ys) = @_;
+    if ($xs eq '+') {
+        if ($ys eq '+') {
+            return $class -> _acmp($xa, $ya);
+        } else {
+            return 1;
+        }
+    } else {
+        if ($ys eq '+') {
+            return -1;
+        } else {
+            return $class -> _acmp($ya, $xa);
+        }
+    }
+}
+
 sub _len {
     my ($class, $x) = @_;
     CORE::length($class -> _str($x));
@@ -710,19 +728,22 @@ sub _log_int {
 
     # X == 1 => 0 (is exact)
     if ($class -> _is_one($x)) {
-        return $class -> _zero(), 1;
+        return $class -> _zero(), 1 if wantarray;
+        return $class -> _zero();
     }
 
     my $cmp = $class -> _acmp($x, $base);
 
     # X == BASE => 1 (is exact)
     if ($cmp == 0) {
-        return $class -> _one(), 1;
+        return $class -> _one(), 1 if wantarray;
+        return $class -> _one();
     }
 
     # 1 < X < BASE => 0 (is truncated)
     if ($cmp < 0) {
-        return $class -> _zero(), 0;
+        return $class -> _zero(), 0 if wantarray;
+        return $class -> _zero();
     }
 
     my $y;
@@ -745,10 +766,6 @@ sub _log_int {
     my $trial = $class -> _pow($class -> _copy($base), $y);
     my $acmp  = $class -> _acmp($trial, $x);
 
-    # Did we get the exact result?
-
-    return $y, 1 if $acmp == 0;
-
     # Too small?
 
     while ($acmp < 0) {
@@ -765,8 +782,93 @@ sub _log_int {
         $acmp  = $class -> _acmp($trial, $x);
     }
 
-    return $y, 1 if $acmp == 0;         # result is exact
-    return $y, 0;                       # result is too small
+    return wantarray ? ($y, 1) : $y if $acmp == 0;      # result is exact
+    return wantarray ? ($y, 0) : $y;                    # result is too small
+}
+
+sub _ilog2 {
+    my ($class, $x) = @_;
+
+    return if $class -> _is_zero($x);
+
+    my $str = $class -> _to_hex($x);
+
+    # First do the bits in all but the most significant hex digit.
+
+    my $y = $class -> _new(length($str) - 1);
+    $y = $class -> _mul($y, $class -> _new(4));
+
+    # Now add the number of bits in the most significant hex digit.
+
+    my $n = int log(hex(substr($str, 0, 1))) / log(2);
+    $y = $class -> _add($y, $class -> _new($n));
+    return $y unless wantarray;
+
+    my $pow2 = $class -> _lsft($class -> _one(), $y, 2);
+    my $is_exact = $class -> _acmp($x, $pow2) == 0 ? 1 : 0;
+    return $y, $is_exact;
+}
+
+sub _ilog10 {
+    my ($class, $x) = @_;
+
+    return if $class -> _is_zero($x);
+
+    my $str = $class -> _str($x);
+    my $len = length($str);
+    my $y = $class -> _new($len - 1);
+    return $y unless wantarray;
+
+    #my $pow10 = $class -> _1ex($y);
+    #my $is_exact = $class -> _acmp($x, $pow10) ? 1 : 0;
+
+    my $is_exact = $str =~ /^10*$/ ? 1 : 0;
+    return $y, $is_exact;
+}
+
+sub _clog2 {
+    my ($class, $x) = @_;
+
+    return if $class -> _is_zero($x);
+
+    my $str = $class -> _to_hex($x);
+
+    # First do the bits in all but the most significant hex digit.
+
+    my $y = $class -> _new(length($str) - 1);
+    $y = $class -> _mul($y, $class -> _new(4));
+
+    # Now add the number of bits in the most significant hex digit.
+
+    my $n = int log(hex(substr($str, 0, 1))) / log(2);
+    $y = $class -> _add($y, $class -> _new($n));
+
+    # $y is now 1 too small unless $y is an exact power of 2.
+
+    my $pow2 = $class -> _lsft($class -> _one(), $y, 2);
+    my $is_exact = $class -> _acmp($x, $pow2) == 0 ? 1 : 0;
+    $y = $class -> _inc($y) if $is_exact == 0;
+    return $y, $is_exact if wantarray;
+    return $y;
+}
+
+sub _clog10 {
+    my ($class, $x) = @_;
+
+    return if $class -> _is_zero($x);
+
+    my $str = $class -> _str($x);
+    my $len = length($str);
+
+    if ($str =~ /^10*$/) {
+        my $y = $class -> _new($len - 1);
+        return $y, 1 if wantarray;
+        return $y;
+    }
+
+    my $y = $class -> _new($len);
+    return $y, 0 if wantarray;
+    return $y;
 }
 
 sub _sqrt {
@@ -1822,17 +1924,19 @@ sub _modpow {
                                         : $class -> _zero();
     }
 
-    #  $num = $class -> _mod($num, $mod);   # this does not make it faster
+    # We could do the following, but it doesn't actually save any time. The
+    # _copy() is needed in case $num and $mod are the same object.
+
+    $num = $class -> _mod($class -> _copy($num), $mod);
 
     my $acc = $class -> _copy($num);
     my $t   = $class -> _one();
 
-    my $expbin = $class -> _as_bin($exp);
-    $expbin =~ s/^0b//;
+    my $expbin = $class -> _to_bin($exp);
     my $len = length($expbin);
 
-    while (--$len >= 0) {
-        if (substr($expbin, $len, 1) eq '1') {
+    while ($len--) {
+        if (substr($expbin, $len, 1) eq '1') {  # if odd
             $t = $class -> _mul($t, $acc);
             $t = $class -> _mod($t, $mod);
         }
@@ -2303,6 +2407,47 @@ has two output arguments, the OBJECT and a STATUS. The STATUS is Perl scalar;
 it is 1 if OBJ is the exact result, 0 if the result was truncted to give OBJ,
 and undef if it is unknown whether OBJ is the exact result.
 
+=item CLASS-E<gt>_ilog2(OBJ)
+
+Returns the base 2 logarithm of OBJ rounded downwards to the nearest integer,
+i.e., C<int(log2(OBJ))>. In list context, this method returns two output
+arguments, the OBJECT and a STATUS. The STATUS is Perl scalar; it is 1 if OBJ
+is the exact result, 0 if the result was truncted to give OBJ, and undef if it
+is unknown whether OBJ is the exact result.
+
+This method is equivalent to the more general method _log_int() when it is used
+with base 2 argument, but _ilog2() method might be faster.
+
+=item CLASS-E<gt>_ilog10(OBJ)
+
+Returns the base 10 logarithm of OBJ rounded downwards to the nearest integer,
+i.e., C<int(log2(OBJ))>. In list context, this method returns two output
+arguments, the OBJECT and a STATUS. The STATUS is Perl scalar; it is 1 if OBJ
+is the exact result, 0 if the result was truncted to give OBJ, and undef if it
+is unknown whether OBJ is the exact result.
+
+This method is equivalent to the more general method _log_int() when it is used
+with base 10 argument, but _ilog10() method might be faster.
+
+Also, the output from _ilog10() is always 1 smaller than the output from
+_len().
+
+=item CLASS-E<gt>_clog2(OBJ)
+
+Returns the base 2 logarithm of OBJ rounded upwards to the nearest integer,
+i.e., C<ceil(log2(OBJ))>. In list context, this method returns two output
+arguments, the OBJECT and a STATUS. The STATUS is Perl scalar; it is 1 if OBJ
+is the exact result, 0 if the result was truncted to give OBJ, and undef if it
+is unknown whether OBJ is the exact result.
+
+=item CLASS-E<gt>_clog10(OBJ)
+
+Returns the base 10 logarithm of OBJ rounded upnwards to the nearest integer,
+i.e., C<ceil(log2(OBJ))>. In list context, this method returns two output
+arguments, the OBJECT and a STATUS. The STATUS is Perl scalar; it is 1 if OBJ
+is the exact result, 0 if the result was truncted to give OBJ, and undef if it
+is unknown whether OBJ is the exact result.
+
 =item CLASS-E<gt>_gcd(OBJ1, OBJ2)
 
 Returns the greatest common divisor of OBJ1 and OBJ2.
@@ -2388,6 +2533,11 @@ Return a true value if OBJ is an even integer, and a false value otherwise.
 Compare OBJ1 and OBJ2 and return -1, 0, or 1, if OBJ1 is numerically less than,
 equal to, or larger than OBJ2, respectively.
 
+=item CLASS-E<gt>_scmp(OBJ1, SIGN1, OBJ2, SIGN2)
+
+Compare OBJ1 and OBJ2 and return -1, 0, or 1, if OBJ1 is numerically less than,
+equal to, or larger than OBJ2, respectively.
+
 =back
 
 =head3 String conversion
